/*********************
I2C驱动代码 (I2C.c)
*********************/
#include "STC8H.H"
#include <intrins.h>

#define u8  unsigned char
#define u16 unsigned int

// I2C引脚定义
sbit I2C_SDA = P1^0;  // P1.0
sbit I2C_SCL = P1^1;  // P1.1

// I2C起始信号
void I2C_Start(void) {
    I2C_SDA = 1;
    I2C_SCL = 1;
    _nop_(); _nop_();
    I2C_SDA = 0;
    _nop_(); _nop_();
    I2C_SCL = 0;
}

// I2C停止信号
void I2C_Stop(void) {
    I2C_SDA = 0;
    I2C_SCL = 1;
    _nop_(); _nop_();
    I2C_SDA = 1;
}

// 发送应答
void I2C_Ack(u8 ack) {
    I2C_SDA = ack;
    I2C_SCL = 1;
    _nop_(); _nop_();
    I2C_SCL = 0;
    I2C_SDA = 1;
}

// 发送单字节
void I2C_SendByte(u8 dat) {
    u8 i;
    u16 timeout = 1000;
    
    // 发送数据位
    for(i=0; i<8; i++) {
        I2C_SDA = (dat & 0x80) ? 1 : 0;
        dat <<= 1;
        I2C_SCL = 1;
        _nop_(); _nop_();
        I2C_SCL = 0;
    }
    
    // 等待ACK
    I2C_SDA = 1;
    I2C_SCL = 1;
    while(I2C_SDA && timeout--) _nop_();
    I2C_SCL = 0;
}

// 接收单字节
u8 I2C_RecvByte(void) {
    u8 i, dat = 0;
    I2C_SDA = 1;
    for(i=0; i<8; i++) {
        dat <<= 1;
        I2C_SCL = 1;
        _nop_(); _nop_();
        dat |= I2C_SDA;
        I2C_SCL = 0;
    }
    return dat;
}