#include "STC8H.H"
#include "intrins.h"
#include <stdio.h>
#include <math.h>

#define u8 unsigned char
#define u16 unsigned int
#define u32 unsigned long

// ------ 新增宏定义 ------
#define RELAY_PORT0 0   // 控制K1-K8的端口
#define RELAY_PORT4 1   // 控制K9-K16的端口
#define OPAMP_SINGLE 0  // 运放类型
#define OPAMP_DUAL   1
#define OPAMP_QUAD   2

// ------ 新增全局变量 ------
xdata u8 OpampType = 0; // 当前运放类型（默认单运放）
bit RST_Flag = 0;       // 复位标志

// ADS1115和LCD原有定义保持不变...

/*********************
函数声明新增部分
*********************/
void RelayControl(u8 port, u8 state);  // 修改继电器控制函数
void CheckButtons();                   // 新增按键检测函数

/*********************
主函数修改
*********************/
void main()
{
    // 新增端口初始化
    P0M0 = 0xFF;  // P0推挽输出（驱动K1-K8）
    P0M1 = 0x00;
    P5M0 = 0x00;  // P5.4准双向（RST按键）
    P5M1 = 0x00;
    
    // 原有初始化保持不变...
    
    while(1)
    {
        CheckButtons(); // 新增按键检测
        
        if(RST_Flag) {  // 处理复位
            LCD_Clear();
            Vos = IB = Aol = PSRR = VoMax = 0;
            RelayControl(RELAY_PORT0, 0x00);
            RelayControl(RELAY_PORT4, 0x00);
            RST_Flag = 0;
        }
        
        // 原按键S1检测修改为复合功能
        if(P70 == 0) 
        {
            Delay1ms(10);
            if(P70 == 0) 
            {
                // 根据运放类型执行测试
                switch(OpampType) {
                    case OPAMP_SINGLE:
                        MeasureVos();  // 单运放测试流程
                        MeasureIB();
                        break;
                    case OPAMP_DUAL:
                        // 双运放测试流程（需补充）
                        break;
                    case OPAMP_QUAD:
                        // 四运放测试流程（需补充）
                        break;
                }
                DisplayResults();
                while(P70 == 0);
            }
        }
    }
}

/*********************
新增按键检测函数
*********************/
void CheckButtons()
{
    // 检测运放类型切换按键（S2）
    static bit S2_Last = 1;
    if(P71 == 0 && S2_Last) {
        Delay1ms(20);
        if(P71 == 0) {
            if(++OpampType > 2) OpampType = 0;
            // 根据运放类型设置基础继电器
            switch(OpampType) {
                case OPAMP_SINGLE: 
                    RelayControl(RELAY_PORT0, 0x01); // 仅K1闭合
                    break;
                case OPAMP_DUAL:
                    RelayControl(RELAY_PORT0, 0x03); // K1-K2闭合
                    break;
                case OPAMP_QUAD:
                    RelayControl(RELAY_PORT0, 0xFF); // 全闭合
                    break;
            }
        }
        while(P71 == 0); // 等待释放
    }
    S2_Last = P71;

    // 检测RST按键（P5.4）
    static bit RST_Last = 1;
    if(P54 == 0 && RST_Last) {
        Delay1ms(20);
        if(P54 == 0) RST_Flag = 1;
        while(P54 == 0);
    }
    RST_Last = P54;
}

/*********************
修改后的继电器控制函数
参数：port-0控制P0(K1-K8),1控制P4(K9-K16)
*********************/
void RelayControl(u8 port, u8 state)
{
    if(port == RELAY_PORT0)
        P0 = state;  // 控制K1-K8
    else
        P4 = state;  // 控制K9-K16
}

/*********************
修改显示函数
*********************/
void DisplayResults()
{
    char buf[32];
    const char *OpampTypeStr[] = {"Single"," Dual "," Quad "};
    
    // 显示运放类型
    sprintf(buf, "OP:%s", OpampTypeStr[OpampType]);
    LCD_ShowStr(64, 6, buf);
    
    // 原有显示内容保持不变...
}

/*********************
示例：修改后的测量函数
*********************/
void MeasureVos()
{
    // 根据运放类型配置不同继电器组合
    switch(OpampType) {
        case OPAMP_SINGLE:
            RelayControl(RELAY_PORT4, 0x23); // K9,10,14
            break;
        case OPAMP_DUAL:
            RelayControl(RELAY_PORT4, 0x27); // 不同组合
            break;
        case OPAMP_QUAD:
            RelayControl(RELAY_PORT4, 0x2F);
            break;
    }
    
    // 原有测量流程保持不变...
}